//
//  VuzeMacDockPlugin.java
//
// Copyright (c) 2009 Marc Haisenko <marc@darkdust.net>
// All rights reserved.
//
// Redistribution and use in source and binary forms, with or without
// modification, are permitted provided that the following conditions are met:
//
// Redistributions of source code must retain the above copyright notice, this
// list of conditions and the following disclaimer.
//
// Redistributions in binary form must reproduce the above copyright notice,
// this list of conditions and the following disclaimer in the documentation
// and/or other materials provided with the distribution.
//
// Neither the name of the Marc Haisenko nor the names of its contributors may
// be used to endorse or promote products derived from this software without
// specific prior written permission.
//
// THIS SOFTWARE IS PROVIDED BY THE COPYRIGHT HOLDERS AND CONTRIBUTORS "AS IS"
// AND ANY EXPRESS OR IMPLIED WARRANTIES, INCLUDING, BUT NOT LIMITED TO, THE
// IMPLIED WARRANTIES OF MERCHANTABILITY AND FITNESS FOR A PARTICULAR PURPOSE
// ARE DISCLAIMED. IN NO EVENT SHALL THE COPYRIGHT HOLDER OR CONTRIBUTORS BE
// LIABLE FOR ANY DIRECT, INDIRECT, INCIDENTAL, SPECIAL, EXEMPLARY, OR CONSE-
// QUENTIAL DAMAGES (INCLUDING, BUT NOT LIMITED TO, PROCUREMENT OF SUBSTITUTE
// GOODS OR SERVICES; LOSS OF USE, DATA, OR PROFITS; OR BUSINESS INTERRUPTION)
// HOWEVER CAUSED AND ON ANY THEORY OF LIABILITY, WHETHER IN CONTRACT, STRICT
// LIABILITY, OR TORT (INCLUDING NEGLIGENCE OR OTHERWISE) ARISING IN ANY WAY
// OUT OF THE USE OF THIS SOFTWARE, EVEN IF ADVISED OF THE POSSIBILITY OF
// SUCH DAMAGE.

package net.darkdust.VuzeMacDockPlugin;

import java.net.URL;
import java.util.zip.ZipEntry;
import java.util.zip.ZipFile;
import java.io.File;
import java.io.FileOutputStream;
import java.io.FileWriter;
import java.io.InputStream;
import org.gudy.azureus2.plugins.Plugin;
import org.gudy.azureus2.plugins.PluginException;
import org.gudy.azureus2.plugins.PluginInterface;
import org.gudy.azureus2.plugins.UnloadablePlugin;
import org.gudy.azureus2.plugins.download.DownloadManager;
import org.gudy.azureus2.plugins.download.DownloadManagerStats;
import org.gudy.azureus2.plugins.logging.LoggerChannel;
import org.gudy.azureus2.plugins.utils.UTTimer;
import org.gudy.azureus2.plugins.utils.UTTimerEvent;
import org.gudy.azureus2.plugins.utils.UTTimerEventPerformer;

public class VuzeMacDockPlugin implements Plugin, UnloadablePlugin, UTTimerEventPerformer
{	
	// Instance of DownloadManager to query the speed.
	static DownloadManager downloadManager;
	// Instance of LoggerChannel for this plugin.
	static LoggerChannel loggerChannel;
	// Timer that triggers the icon update.
	static UTTimer timer;
	// Whether the JNI library was loaded.
	static boolean libraryLoaded;
	// Whether the overlay image was already loaded.
	static boolean imageLoaded;
	// Whether the plugin is currently running.
	static boolean active;
	
    static
	{
		downloadManager = null;
		loggerChannel = null;
		libraryLoaded = false;
		imageLoaded = false;
		active = false;
		
		// Load native JNI library from JAR... kudos to Thomas Robinson.
		// http://tlrobinson.net/blog/2009/03/09/embedding-and-loading-a-jni-library-from-a-jar/

		try
		{
			// get the class object for this class, and get the location of it
			final Class c = VuzeMacDockPlugin.class;
			final URL location = c.getProtectionDomain().getCodeSource().getLocation();
			
			// jars are just zip files, get the input stream for the lib
			ZipFile zf = new ZipFile(location.getPath());
			InputStream in = zf.getInputStream(zf.getEntry("libVuzeMacDockPlugin.jnilib"));
			
			// create a temp file and an input stream for it
			File f = File.createTempFile("JARLIB-", "-libVuzeMacDockPlugin.jnilib");
			FileOutputStream out = new FileOutputStream(f);
			
			// copy the lib to the temp file
			byte[] buf = new byte[1024];
			int len;
			while ((len = in.read(buf)) > 0)
			{
				out.write(buf, 0, len);
			}
			
			in.close();
			out.close();
			
			// load the lib specified by it’s absolute path and delete it
			System.load(f.getAbsolutePath());
			f.delete();
			
			libraryLoaded = true;
		}
		catch (Exception e)
		{
			System.err.println("VuzeMacDockPlugin: Error while loading the JNI native library:");
			System.err.println(e.getMessage());
			e.printStackTrace(System.err);
		}
    }

	// Activate the plugin.
	public void initialize(PluginInterface pluginInterface) throws PluginException
	{
		if (loggerChannel == null)
			loggerChannel = pluginInterface.getLogger().getChannel("VuzeMacDockPlugin");
		
		if (!imageLoaded)
		{
			// Load the overlay image.
			try
			{
				// get the class object for this class, and get the location of it
				final Class c = VuzeMacDockPlugin.class;
				final URL location = c.getProtectionDomain().getCodeSource().getLocation();
				int size;
				
				// jars are just zip files, get the input stream for the image
				ZipFile zf = new ZipFile(location.getPath());
				ZipEntry ze = zf.getEntry("Overlay.png");
				InputStream in = zf.getInputStream(ze);
				
				// read complete image
				byte[] buf = new byte[(int) ze.getSize()];
				if ((size = in.read(buf)) != ze.getSize())
				{
					loggerChannel.logAlert(LoggerChannel.LT_ERROR, "Error while reading the overlay image: read size " +
										   size + " does not match expected size " +
										   ze.getSize() + " ! Plugin not activated !");
				}
				else
				{
					if (initializeDockIcon(buf, size) == 0)
					{
						loggerChannel.logAlert(LoggerChannel.LT_ERROR, "Could not initialize JNI part ! Plugin not activated !");
					}
					else
					{
						// Weehoo, we're set up !
						imageLoaded = true;
					}
				}
				
				in.close();
			}
			catch (Exception e)
			{
				loggerChannel.logAlert(LoggerChannel.LT_ERROR, "Could not load overlay image: " +
									   e.getMessage() + ". Plugin not activated !");
			}
		}
		
		if (imageLoaded && timer == null)
		{
			if (downloadManager == null)
				downloadManager = pluginInterface.getDownloadManager();
			
			timer = pluginInterface.getUtilities().createTimer("VuzeMacDockPluginTimer");
			timer.addPeriodicEvent(2000, this);
			
			loggerChannel.log(LoggerChannel.LT_INFORMATION, "Plugin active.");
		}
	}
	
	// Deactivate the plugin.
	public void unload()
	{
		if (timer != null)
		{
			timer.destroy();
			timer = null;
			loggerChannel.log(LoggerChannel.LT_INFORMATION, "Plugin deactivated.");
		}
	}
	
	// This is what the plugin is all about: get current download/upload rate and update the dock icon.
	public void perform(UTTimerEvent event)
	{
		DownloadManagerStats stats;
		
		stats = downloadManager.getStats();
		updateDockIcon(stats.getDataReceiveRate(), stats.getDataSendRate());
    }

	// The native methods:
    native int updateDockIcon(long up, long down);
	native int initializeDockIcon(byte[] data, long size);
}
